@@ -278,6 +278,80 @@ class PyPackage(pyobjects.PyPackage):
         return self
 
 
+class _AnnAssignVisitor(object):
+
+    def __init__(self, scope_visitor):
+        self.scope_visitor = scope_visitor
+        self.assigned_ast = None
+        self.type_hint = None
+
+    def _AnnAssign(self, node):
+        self.assigned_ast = node.value
+        self.type_hint = node.annotation
+
+        ast.walk(node.target, self)
+
+    def _assigned(self, name, assignment=None):
+        self.scope_visitor._assigned(name, assignment)
+
+    def _Name(self, node):
+        assignment = pynames.AssignmentValue(self.assigned_ast,
+                                             assign_type=True,
+                                             type_hint=self.type_hint)
+        self._assigned(node.id, assignment)
+
+    def _Tuple(self, node):
+        names = astutils.get_name_levels(node)
+        for name, levels in names:
+            assignment = None
+            if self.assigned_ast is not None:
+                assignment = pynames.AssignmentValue(self.assigned_ast, levels)
+            self._assigned(name, assignment)
+
+    def _Annotation(self, node):
+        pass
+
+    def _Attribute(self, node):
+        pass
+
+    def _Subscript(self, node):
+        pass
+
+    def _Slice(self, node):
+        pass
+
+
+class _ExpressionVisitor(object):
+    def __init__(self, scope_visitor):
+        self.scope_visitor = scope_visitor
+
+    def _assigned(self, name, assignment=None):
+        self.scope_visitor._assigned(name, assignment)
+
+    def _GeneratorExp(self, node):
+        for child in ['elt', 'key', 'value']:
+            if hasattr(node, child):
+                ast.walk(getattr(node, child), self)
+        for comp in node.generators:
+            ast.walk(comp.target, _AssignVisitor(self))
+            ast.walk(comp, self)
+            for if_ in comp.ifs:
+                ast.walk(if_, self)
+
+    def _ListComp(self, node):
+        self._GeneratorExp(node)
+
+    def _SetComp(self, node):
+        self._GeneratorExp(node)
+
+    def _DictComp(self, node):
+        self._GeneratorExp(node)
+
+    def _NamedExpr(self, node):
+        ast.walk(node.target, _AssignVisitor(self))
+        ast.walk(node.value, self)
+
+
 class _AssignVisitor(object):
 
     def __init__(self, scope_visitor):
@@ -288,6 +362,7 @@ class _AssignVisitor(object):
         self.assigned_ast = node.value
         for child_node in node.targets:
             ast.walk(child_node, self)
+        ast.walk(node.value, _ExpressionVisitor(self.scope_visitor))
 
     def _assigned(self, name, assignment=None):
         self.scope_visitor._assigned(name, assignment)
@@ -316,7 +391,7 @@ class _AssignVisitor(object):
         pass
 
 
-class _ScopeVisitor(object):
+class _ScopeVisitor(_ExpressionVisitor):
 
     def __init__(self, pycore, owner_object):
         self.pycore = pycore
@@ -347,16 +422,25 @@ class _ScopeVisitor(object):
                     def _eval(type_=type_, arg=arg):
                         return type_.get_property_object(
                             arguments.ObjectArguments([arg]))
+
+                    lineno = utils.guess_def_lineno(self.get_module(), node)
+
                     self.names[node.name] = pynames.EvaluatedName(
-                        _eval, module=self.get_module(), lineno=node.lineno)
+                        _eval, module=self.get_module(), lineno=lineno)
                     break
         else:
             self.names[node.name] = pynames.DefinedName(pyfunction)
         self.defineds.append(pyfunction)
 
+    def _AsyncFunctionDef(self, node):
+        return self._FunctionDef(node)
+
     def _Assign(self, node):
         ast.walk(node, _AssignVisitor(self))
 
+    def _AnnAssign(self, node):
+        ast.walk(node, _AnnAssignVisitor(self))
+
     def _AugAssign(self, node):
         pass
 
@@ -366,6 +450,9 @@ class _ScopeVisitor(object):
         for child in node.body + node.orelse:
             ast.walk(child, self)
 
+    def _AsyncFor(self, node):
+        return self._For(node)
+
     def _assigned(self, name, assignment):
         pyname = self.names.get(name, None)
         if pyname is None:
@@ -376,7 +463,7 @@ class _ScopeVisitor(object):
             self.names[name] = pyname
 
     def _update_evaluated(self, targets, assigned,
-                          evaluation='', eval_type=False):
+                          evaluation='', eval_type=False, type_hint=None):
         result = {}
         if isinstance(targets, str):
             assignment = pynames.AssignmentValue(assigned, [],
@@ -398,6 +485,9 @@ class _ScopeVisitor(object):
         for child in node.body:
             ast.walk(child, self)
 
+    def _AsyncWith(self, node):
+        return self._With(node)
+
     def _excepthandler(self, node):
         node_name_type = str if pycompat.PY3 else ast.Name
         if node.name is not None and isinstance(node.name, node_name_type):
